package RPM::Source::Dependency::Analyzer::Sourcedep::Commandline;

use strict;
use warnings;
#use File::Basename;
use Carp;

use RPM::Source::Dependency::Analyzer::PerlUtils qw(_perl_pkg2dep _perl_req2dep _perl_expr2dep);

our $verbose=0;
our $VERSION=0.002;

my $sourcedep_opts={
    '--add-pkgconfig' => \&plugin_sourcedep_opt_add_pkgconfig,
    '--add-devel-lib' => \&plugin_sourcedep_opt_add_devel_lib,
    '--add-raw-lib' => \&plugin_sourcedep_opt_add_raw_lib,
    '--add-typelib' => \&plugin_sourcedep_opt_add_typelib,
    '--add-header' => \&plugin_sourcedep_opt_add_header,
    '--add-gir' => \&plugin_sourcedep_opt_add_gir,
    '--add-vapi-or-gir' => \&plugin_sourcedep_opt_add_vapi_or_gir,
    '--add-cmake' => \&plugin_sourcedep_opt_add_cmake,
    '--add-path' => \&plugin_sourcedep_opt_add_path,
    '--add-bin-or-path' => \&plugin_sourcedep_opt_add_bin_or_path,
    '--add-perl-module' => \&plugin_sourcedep_opt_add_perl_module,
    '--add-perl-expression' => \&plugin_sourcedep_opt_add_perl_expression,
};

sub get_options_list {
    return sort keys(%$sourcedep_opts);
}

sub process_argv {
    my ($distroinfo,$storage,$cmd,@argvs)=@_;
    my $opt_handler;
    my @args;
    while (@argvs) {
	my $opt=shift @argvs;
	if ($opt!~/^--/) {
	    push @args, $opt;
	} else {
	    if ($opt_handler) {
		map { &$opt_handler($distroinfo,$storage,$_) || return 0 } @args;
		@args=();
	    }
	    $opt_handler=$sourcedep_opts->{$opt};
	    if (!$opt_handler) {
		warn 'Batch: ', $cmd, ': unsupported option ',$opt,' in ',join(' ',@argvs),"\n";
		return 0;
	    }
	}
    }
    map { &$opt_handler($distroinfo,$storage,$_) || return 0 } @args;
    return 1;
}

sub plugin_sourcedep_opt_add_devel_lib {
    my ($distroinfo,$storage,$arg)=@_;
    $storage->add_libs_req($_);
    return 1;
}

sub plugin_sourcedep_opt_add_raw_lib {
    my ($distroinfo,$storage,$arg)=@_;
    # we do not have special distrodb for raw libs for now :(
    # libznz.so.2 -> libznz.so.2()(64bit)
    my @pkgs=$distroinfo->translate('provides',$arg.'()(64bit)');
    $storage->add_pkgs_req($pkgs[0]) if @pkgs;
    return 1;
}

sub plugin_sourcedep_opt_add_typelib {
    my ($distroinfo,$storage,$arg)=@_;
    # we do not have special distrodb for typelib for now :(
    my $provides='typelib('.$arg.')';
    my @pkgs=$distroinfo->translate('provides',$provides);
    $storage->add_pkgs_req($provides) if @pkgs;
    return 1;
}

sub plugin_sourcedep_opt_add_gir {
    my ($distroinfo,$storage,$arg)=@_;
    $storage->add_girs_req($arg);
    return 1;
}

sub plugin_sourcedep_opt_add_vapi_or_gir {
    my ($distroinfo,$storage,$arg)=@_;
    my @pkgs=$distroinfo->translate('vapi',$arg);
    if (@pkgs) {
	$storage->add_vapi_req($arg);
    } else {
	$storage->add_girs_req($arg);
    }
    return 1;
}

sub plugin_sourcedep_opt_add_header {
    my ($distroinfo,$storage,$arg)=@_;
    $storage->add_hdrs_req($arg);
    return 1;
}

sub plugin_sourcedep_opt_add_pkgconfig {
    my ($distroinfo,$storage,$arg)=@_;
    $storage->add_pkgconf_req($arg);
    return 1;
}

sub plugin_sourcedep_opt_add_cmake {
    my ($distroinfo,$storage,$arg)=@_;
    $storage->add_cmake_req($arg);
    return 1;
}

sub plugin_sourcedep_opt_add_path {
    my ($distroinfo,$storage,$arg)=@_;
    $storage->add_path_req($arg);
    return 1;
}

sub plugin_sourcedep_opt_add_bin_or_path {
    my ($distroinfo,$storage,$arg)=@_;
    if (substr($arg,0,1) eq '/') {
	my @pkgs=$distroinfo->translate('bin',$arg);
	# to avoid ambiguous /usr/bin/coverage and co.
	$storage->add_pkgs_req($pkgs[0]) if @pkgs==1;
    } else {
	$storage->add_path_req($arg);
    }
    return 1;
}

sub plugin_sourcedep_opt_add_perl_module {
    my ($distroinfo,$storage,$arg)=@_;
    $storage->add_perl_req(&_perl_pkg2dep($arg));
    return 1;
}

sub plugin_sourcedep_opt_add_perl_expression {
    my ($distroinfo,$storage,$arg)=@_;
    $storage->add_perl_req(&_perl_expr2dep($arg));
    return 1;
}

1;
